/****************************************************************************
	INCLUDE FILES
****************************************************************************/

#include <dos.h>
#include <stdio.h>

/****************************************************************************
	TYPEDEFS AND DEFINES
****************************************************************************/

#define 	DEBUG 1

#define FALSE		0
#define TRUE		!FALSE

// register definitions for 8259 programmable interrupt controller

#define INT_0_CTRL_ADDR	0x20	// 8259 master (0) interrupt control register
#define INT_0_MASK_ADDR	0x21	// 8259 master (0) interrupt mask register
#define INT_1_CTRL_ADDR	0xA0	// 8259 slave  (1) interrupt control register
#define INT_1_MASK_ADDR	0xA1	// 8259 slave  (1) interrupt mask register   

#define END_OF_INT		0x20	// generic end of interrupt mask

/****************************************************************************
	GLOBAL VARIABLES
****************************************************************************/

#if DEBUG
void interrupt(*NewVector)();			// holds our vector address, for debug
#endif

void interrupt(*OldVector)();			// holds the previous vector for INT7

volatile unsigned char ucPort;		// temp storage for port I/O
volatile unsigned char ucCount;		// generic counter storage

/****************************************************************************
	FUNCTION PROTOTYPES
****************************************************************************/

void interrupt myISR( void );

/****************************************************************************
	EXTERNAL DECLARATIONS
****************************************************************************/

/****************************************************************************
	 Function: main
 Description: initialization, main loop, de-initialization
		Passed: none
	  Returns: none
*****************************************************************************/

main()
{

	// Timer1 output is directly connected to IRQ10 inside the 386EX processor

	// IRQ10 calls the interrupt service routine installed at vector 0x72

	//-------------------------------------------------------------------------
	// Setup interrupt vectors so that IRQ10 calls our interrupt service routine
	//-------------------------------------------------------------------------

	OldVector = getvect(0x72);			// get old int. vector to re-install on exit

	disable();							// disable interrupts for next call

	setvect(0x72,myISR);				// set int. vector for our routine

	enable();							// enable system interrupts

	//-------------------------------------------------------------------------
	// Setup Timer 1 for 4 Khz square wave function.
	//-------------------------------------------------------------------------

#if DEBUG

	// IRQ3 is pin B25 on the PC/104 bus, this code will connect the IRQ3 pin
	// to Timer 1 output pin, useful to see if Timer 1 is running and how fast.
	outportb( 0xf832, 0x3f );           
#endif

	outportb( 0x43, 0x76 );					// set Timer1 - binary, Mode 3
	outportb( 0x41, 0xFA );					// set Timer1 to devide by 250
	outportb( 0x41, 0x00 );					// this will yield 4 Khz square wave

	//-------------------------------------------------------------------------
	// setup interrupt controller to enable IRQ10 (interrupt 2 on second 8259)
	//-------------------------------------------------------------------------

	disable();	// don't let interrupts happen during read/modify/write

	ucPort = inportb( INT_1_MASK_ADDR );	// read interrupt mask, 8259(1)

	ucPort &= 0xFB;							// clear mask bit for IRQ10

	outportb( INT_1_MASK_ADDR, ucPort );	// write modified mask 8259(1)

	enable();	// re-enable system interrupts


//	asm int 3;	// uncomment this to jump into debugger

#if DEBUG
	NewVector = getvect(0x72);				// get int. vector for RTC (IRQ7)

	printf( "OldVector=(%p)\n", OldVector );
	printf( "NewVector=(%p)\n", NewVector );
#endif


	while( !kbhit() );	// wait for keypress to exit


	//-------------------------------------------------------------------------
	// setup interrupt controller to disable IRQ10 (interrupt 2 on second 8259)
	//-------------------------------------------------------------------------

	disable(); // don't let interrupts happen during read/modify/write

	ucPort = inportb( INT_1_MASK_ADDR );	// read interrupt mask, 8259(1)

	ucPort |= 0x04;							// clear mask bit for IRQ10

	outportb( INT_1_MASK_ADDR, ucPort );	// write modified mask 8259(1)

	enable();  // re-enable system interrupts

	//-------------------------------------------------------------------------
	// Setup interrupt vectors so that IRQ10 calls old interrupt routine
	//-------------------------------------------------------------------------

	disable();								// disable interrupts

	setvect( 0x72, OldVector );				// set RTC int. vector for our routine

	enable();								// enable system interrupts

}

/****************************************************************************
	 Function: myISR
 Description:
		Passed: none
	  Returns: none
*****************************************************************************/

void interrupt myISR( void )
{

//	asm int 3;	// drop into debugger to show interrupt happened.

	inportb( 0x140 );				// accessing I/0 x140 is user chip select
									// on TS products, use a scope or logic
									// on pin B19 on the PC/104 bus to see if 
									// the interrupt routine is getting called

	inportb( 0x140 );				// do it more so scopes can see this
	inportb( 0x140 );
	inportb( 0x140 );
	inportb( 0x140 );

	//-------------------------------------------------------------------------
	// send end of interrupt to second, then first 8259 interrupt controller
	//-------------------------------------------------------------------------

	outportb( INT_1_CTRL_ADDR, END_OF_INT );
	outportb( INT_0_CTRL_ADDR, END_OF_INT );
}
